@@ -22,6 +22,7 @@
 #include <glib-object.h>
 
 #include "atk.h"
+#include "atkmarshal.h"
 
 /* New GObject properties registered by AtkObject */
 enum
@@ -31,21 +32,16 @@ enum
   PROP_NAME,
   PROP_DESCRIPTION,
   PROP_PARENT,      /* ancestry has changed */
-  PROP_CHILD,       /* a child has been added or removed */
-  PROP_STATE,       /* AtkStateSet for the object has changed */
-  PROP_TEXT,        /* Used only by AtkText implementors */
-  PROP_CARET,       /* Used only by AtkText implementors */
-  PROP_SELECTION,
   PROP_VALUE,
-  PROP_VISIBLE_DATA,
   PROP_ROLE,
+  PROP_LAYER,
+  PROP_MDI_ZORDER,
   PROP_TABLE_CAPTION,
   PROP_TABLE_COLUMN_DESCRIPTION,
   PROP_TABLE_COLUMN_HEADER,
   PROP_TABLE_ROW_DESCRIPTION,
   PROP_TABLE_ROW_HEADER,
   PROP_TABLE_SUMMARY,
-  PROP_MODEL,
   PROP_LAST         /* gobject convention */
 };
 
@@ -53,7 +49,9 @@ enum {
   CHILDREN_CHANGED,
   FOCUS_EVENT,
   PROPERTY_CHANGE,
-
+  STATE_CHANGE,
+  VISIBLE_DATA_CHANGED,
+  
   LAST_SIGNAL
 };
 
@@ -79,6 +77,7 @@ static G_CONST_RETURN gchar*
                                                    (AtkObject       *object);
 static AtkObject*      atk_object_real_get_parent  (AtkObject       *object);
 static AtkRole         atk_object_real_get_role    (AtkObject       *object);
+static AtkLayer        atk_object_real_get_layer   (AtkObject       *object);
 static AtkStateSet*    atk_object_real_ref_state_set
                                                    (AtkObject       *object);
 static void            atk_object_real_set_name    (AtkObject       *object,
@@ -106,23 +105,18 @@ static guint atk_object_signals[LAST_SIGNAL] = { 0, };
 static gpointer parent_class = NULL;
 
 static const gchar* atk_object_name_property_name = "accessible-name";
-static const gchar* atk_object_name_property_state = "accessible-state";
 static const gchar* atk_object_name_property_description = "accessible-description";
-static const gchar* atk_object_name_property_child = "accessible-child";
 static const gchar* atk_object_name_property_parent = "accessible-parent";
-static const gchar* atk_object_name_property_text = "accessible-text";
-static const gchar* atk_object_name_property_caret = "accessible-caret";
-static const gchar* atk_object_name_property_selection = "accessible-selection";
 static const gchar* atk_object_name_property_value = "accessible-value";
-static const gchar* atk_object_name_property_visible = "accessible-visible-data";
 static const gchar* atk_object_name_property_role = "accessible-role";
+static const gchar* atk_object_name_property_component_layer = "accessible-component-layer";
+static const gchar* atk_object_name_property_component_mdi_zorder = "accessible-component-mdi-zorder";
 static const gchar* atk_object_name_property_table_caption = "accessible-table-caption";
 static const gchar* atk_object_name_property_table_column_description = "accessible-table-column-description";
 static const gchar* atk_object_name_property_table_column_header = "accessible-table-column-header";
 static const gchar* atk_object_name_property_table_row_description = "accessible-table-row-description";
 static const gchar* atk_object_name_property_table_row_header = "accessible-table-row-header";
 static const gchar* atk_object_name_property_table_summary = "accessible-table-summary";
-static const gchar* atk_object_name_property_model = "accessible-model";
 
 GType
 atk_object_get_type (void)
@@ -168,6 +162,8 @@ atk_object_class_init (AtkObjectClass *klass)
   klass->get_index_in_parent = NULL;
   klass->ref_relation_set = atk_object_real_ref_relation_set;
   klass->get_role = atk_object_real_get_role;
+  klass->get_layer = atk_object_real_get_layer;
+  klass->get_mdi_zorder = NULL;
   klass->ref_state_set = atk_object_real_ref_state_set;
   klass->set_name = atk_object_real_set_name;
   klass->set_description = atk_object_real_set_description;
@@ -184,6 +180,7 @@ atk_object_class_init (AtkObjectClass *klass)
   klass->children_changed = NULL;
   klass->focus_event = NULL;
   klass->property_change = NULL;
+  klass->visible_data_changed = NULL;
 
   g_object_class_install_property (gobject_class,
                                    PROP_NAME,
@@ -201,23 +198,6 @@ atk_object_class_init (AtkObjectClass *klass)
                                                         "assistive technology access",
                                                         NULL,
                                                         G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-                                   PROP_STATE,
-                                   g_param_spec_int    (atk_object_name_property_state,
-                                                        "Accessible State Set",
-                                                        "The accessible state set of this object "
-                                                        "or its UI component",
-                                                        0,
-                                                        G_MAXINT,
-                                                        0,
-                                                        G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-                                   PROP_CHILD,
-                                   g_param_spec_object (atk_object_name_property_child,
-                                                        "Accessible Child",
-                                                        "Is used to notify that a child has been added or removed ",
-                                                        ATK_TYPE_OBJECT,
-                                                        G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_PARENT,
                                    g_param_spec_object (atk_object_name_property_parent,
@@ -225,29 +205,6 @@ atk_object_class_init (AtkObjectClass *klass)
                                                         "Is used to notify that the parent has changed ",
                                                         ATK_TYPE_OBJECT,
                                                         G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-                                   PROP_TEXT,
-                                   g_param_spec_object (atk_object_name_property_text,
-                                                        "Accessible Text",
-                                                        "Is used to notify that the text has changed ",
-                                                        ATK_TYPE_OBJECT,
-                                                        G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-                                   PROP_CARET,
-                                   g_param_spec_int    (atk_object_name_property_caret,
-                                                        "Accessible Caret",
-                                                        "Is used to notify that the caret position has changed ",
-                                                        0,
-                                                        G_MAXINT,
-                                                        0,
-                                                        G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-                                   PROP_SELECTION,
-                                   g_param_spec_object (atk_object_name_property_selection,
-                                                        "Accessible Selection",
-                                                        "Is used to notify that the selection has changed ",
-                                                        ATK_TYPE_OBJECT,
-                                                        G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_VALUE,
                                    g_param_spec_double (atk_object_name_property_value,
@@ -257,22 +214,33 @@ atk_object_class_init (AtkObjectClass *klass)
                                                         G_MAXDOUBLE,
                                                         0.0,
                                                         G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-                                   PROP_VISIBLE_DATA,
-                                   g_param_spec_object (atk_object_name_property_visible,
-                                                        "Accessible Visible Data",
-                                                        "Is used to notify that the visual appearance of the object has changed ",
-                                                        ATK_TYPE_OBJECT,
-                                                        G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_ROLE,
                                    g_param_spec_int    (atk_object_name_property_role,
                                                         "Accessible Role",
-                                                        "The accessible role this object ",
+                                                        "The accessible role of this object ",
                                                         0,
                                                         G_MAXINT,
                                                         0,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_LAYER,
+                                   g_param_spec_int    (atk_object_name_property_component_layer,
+                                                        "Accessible Layer",
+                                                        "The accessible layer of this object ",
+                                                        0,
+                                                        G_MAXINT,
+                                                        0,
+                                                        G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_MDI_ZORDER,
+                                   g_param_spec_int    (atk_object_name_property_component_mdi_zorder,
+                                                        "Accessible MDI Value",
+                                                        "The accessible MDI value of this object ",
+                                                        G_MININT,
+                                                        G_MAXINT,
+                                                        G_MININT,
+                                                        G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_TABLE_CAPTION,
                                    g_param_spec_string (atk_object_name_property_table_caption,
@@ -315,13 +283,6 @@ atk_object_class_init (AtkObjectClass *klass)
                                                         "Is used to notify that the table summary has changed ",
                                                         ATK_TYPE_OBJECT,
                                                         G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-                                   PROP_MODEL,
-                                   g_param_spec_object (atk_object_name_property_model,
-                                                        "Accessible Model",
-                                                        "Is used to notify that the model for Table or Tree has changed ",
-                                                        ATK_TYPE_OBJECT,
-                                                        G_PARAM_READWRITE));
   /*
    * The signal "children_changed" supports two details:
    * "add" and "remove"
@@ -353,6 +314,29 @@ atk_object_class_init (AtkObjectClass *klass)
                   g_cclosure_marshal_VOID__POINTER,
                   G_TYPE_NONE, 1,
                   G_TYPE_POINTER);
+  /*
+   * The "state_change" signal supports details, one for each accessible 
+   * state type
+   * (see atkstate.c).
+   */
+  atk_object_signals[STATE_CHANGE] =
+    g_signal_new ("state_change",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                  G_STRUCT_OFFSET (AtkObjectClass, state_change),
+                  (GSignalAccumulator) NULL, NULL,
+                  atk_marshal_VOID__STRING_BOOLEAN,
+                  G_TYPE_NONE, 2,
+                  G_TYPE_STRING,
+                  G_TYPE_BOOLEAN);
+  atk_object_signals[VISIBLE_DATA_CHANGED] =
+    g_signal_new ("visible_data_changed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (AtkObjectClass, visible_data_changed),
+                  (GSignalAccumulator) NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 }
 
 static void
@@ -564,6 +548,53 @@ atk_object_get_role (AtkObject *accessible)
     return ATK_ROLE_UNKNOWN;
 }
 
+/**
+ * atk_object_get_layer:
+ * @accessible: an #AtkObject
+ *
+ * Gets the layer of the accessible.
+ *
+ * Returns: an #AtkLayer which is the layer of the accessible
+ **/
+AtkLayer
+atk_object_get_layer (AtkObject *accessible) 
+{
+  AtkObjectClass *klass;
+
+  g_return_val_if_fail (ATK_IS_OBJECT (accessible), ATK_LAYER_INVALID);
+
+  klass = ATK_OBJECT_GET_CLASS (accessible);
+  if (klass->get_layer)
+    return (klass->get_layer) (accessible);
+  else
+    return ATK_LAYER_INVALID;
+}
+
+/**
+ * atk_object_get_mdi_zorder:
+ * @accessible: an #AtkObject
+ *
+ * Gets the zorder of the accessible. The value G_MININT will be returned 
+ * if the layer of the accessible is not ATK_LAYER_MDI.
+ *
+ * Returns: a gint which is the zorder of the accessible, i.e. the depth at 
+ * which the component is shown in relation to other components in the same 
+ * container.
+ **/
+gint
+atk_object_get_mdi_zorder (AtkObject *accessible) 
+{
+  AtkObjectClass *klass;
+
+  g_return_val_if_fail (ATK_IS_OBJECT (accessible), G_MININT);
+
+  klass = ATK_OBJECT_GET_CLASS (accessible);
+  if (klass->get_mdi_zorder)
+    return (klass->get_mdi_zorder) (accessible);
+  else
+    return G_MININT;
+}
+
 /**
  * atk_object_ref_state_set:
  * @accessible: an #AtkObject
@@ -676,7 +707,10 @@ atk_object_set_parent (AtkObject *accessible,
 
   klass = ATK_OBJECT_GET_CLASS (accessible);
   if (klass->set_parent)
-    (klass->set_parent) (accessible, parent);
+    {
+      (klass->set_parent) (accessible, parent);
+      g_object_notify (G_OBJECT (accessible), atk_object_name_property_parent);
+    }
 }
 
 /**
@@ -696,7 +730,10 @@ atk_object_set_role (AtkObject *accessible,
 
   klass = ATK_OBJECT_GET_CLASS (accessible);
   if (klass->set_role)
-    (klass->set_role) (accessible, role);
+    {
+      (klass->set_role) (accessible, role);
+      g_object_notify (G_OBJECT (accessible), atk_object_name_property_role);
+    }
 }
 
 /**
@@ -751,32 +788,19 @@ atk_object_remove_property_change_handler  (AtkObject *accessible,
  * @state: an #AtkState whose state is changed
  * @value : a gboolean which indicates whether the state is being set on or off
  * 
- * Emits a property_change signal for the specified state. 
- * This signal is caught by an #AtkPropertyChangeHandler
+ * Emits a state-change signal for the specified state. 
  **/
 void
 atk_object_notify_state_change (AtkObject *accessible,
                                 AtkState  state,
                                 gboolean  value)
 {
-  AtkPropertyValues  values;
+  G_CONST_RETURN gchar* name;
 
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
-  values.property_name = atk_object_name_property_state;
-  if (value)
-    {
-       g_value_init (&values.new_value, G_TYPE_INT);
-       g_value_set_int (&values.new_value, state);
-    }
-    else
-    {
-       g_value_init (&values.old_value, G_TYPE_INT);
-       g_value_set_int (&values.old_value, state);
-    }
-  g_signal_emit (accessible, atk_object_signals[PROPERTY_CHANGE],
-                 g_quark_from_string (atk_object_name_property_state),
-                 &values, NULL);
+  name = atk_state_type_get_name (state);
+  g_signal_emit (accessible, atk_object_signals[STATE_CHANGE],
+                 g_quark_from_string (name),
+                 name, value, NULL);
 }
 
 /**
@@ -834,8 +858,11 @@ atk_object_real_set_property (GObject      *object,
     case PROP_DESCRIPTION:
       atk_object_set_description (accessible, g_value_get_string (value));
       break;
-    case PROP_STATE:
-      g_print ("This interface does not support setting the state set of an accessible object\n");
+    case PROP_ROLE:
+      atk_object_set_role (accessible, g_value_get_int (value));
+      break;
+    case PROP_PARENT:
+      atk_object_set_parent (accessible, g_value_get_object (value));
       break;
     case PROP_VALUE:
       if (ATK_IS_VALUE (accessible))
@@ -864,11 +891,24 @@ atk_object_real_get_property (GObject      *object,
     case PROP_DESCRIPTION:
       g_value_set_string (value, atk_object_get_description (accessible));
       break;
+    case PROP_ROLE:
+      g_value_set_int (value, atk_object_get_role (accessible));
+      break;
+    case PROP_LAYER:
+      g_value_set_int (value, atk_object_get_role (accessible));
+      break;
+    case PROP_MDI_ZORDER:
+      g_value_set_int (value, atk_object_get_mdi_zorder (accessible));
+      break;
+    case PROP_PARENT:
+      g_value_set_object (value, atk_object_get_parent (accessible));
+      break;
     case PROP_VALUE:
       if (ATK_IS_VALUE (accessible))
         atk_value_get_current_value (ATK_VALUE (accessible), value);
       break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
@@ -921,6 +961,12 @@ atk_object_real_get_role (AtkObject       *object)
   return object->role;
 }
 
+static AtkLayer
+atk_object_real_get_layer (AtkObject       *object)
+{
+  return object->layer;
+}
+
 static AtkStateSet*
 atk_object_real_ref_state_set (AtkObject *accessible) 
 {
@@ -1016,7 +1062,7 @@ atk_object_notify (GObject     *obj,
   AtkPropertyValues values = { 0, };
 
   g_value_init (&values.new_value, pspec->value_type);
-  g_object_get_property(obj, pspec->name, &values.new_value);
+  g_object_get_property (obj, pspec->name, &values.new_value);
   values.property_name = pspec->name;
   g_signal_emit (obj, atk_object_signals[PROPERTY_CHANGE],
                  g_quark_from_string (pspec->name),
