@@ -104,6 +104,7 @@ extern "C" {
  *@ATK_ROLE_TOOL_BAR: A bar or palette usually composed of push buttons or toggle buttons
  *@ATK_ROLE_TOOL_TIP: An object that provides information about another object
  *@ATK_ROLE_TREE: An object used to represent hierarchical information to the user
+ *@ATK_ROLE_TREE_TABLE: An object capable of expanding and collapsing rows as well as showing multiple columns of data
  *@ATK_ROLE_UNKNOWN: The object contains some Accessible information, but its role is not known
  *@ATK_ROLE_VIEWPORT: An object usually used in a scroll pane
  *@ATK_ROLE_WINDOW: A top level window with no title or border
@@ -178,6 +179,7 @@ typedef enum
   ATK_ROLE_TOOL_BAR,
   ATK_ROLE_TOOL_TIP,
   ATK_ROLE_TREE,
+  ATK_ROLE_TREE_TABLE,
   ATK_ROLE_UNKNOWN,
   ATK_ROLE_VIEWPORT,
   ATK_ROLE_WINDOW,
@@ -186,6 +188,28 @@ typedef enum
 
 AtkRole                  atk_role_register        (const gchar *name);
 
+/**
+ *AtkLayer:
+ *@ATK_LAYER_INVALID: The object does not have a layer
+ *@ATK_LAYER_BACKGROUND: This layer is reserved for the desktop background
+ *@ATK_LAYER_CANVAS: This layer is used for Canvas components
+ *@ATK_LAYER_WIDGET: This layer is normally used for components
+ *@ATK_LAYER_MDI: This layer is used for layered components
+ *@ATK_LAYER_POPUP: This layer is used for popup components, such as menus
+ *@ATK_LAYER_OVERLAY: This layer is reserved for future use.
+ *
+ * Describes the layer of a component
+ **/
+typedef enum
+{
+  ATK_LAYER_INVALID,
+  ATK_LAYER_BACKGROUND,
+  ATK_LAYER_CANVAS,
+  ATK_LAYER_WIDGET,
+  ATK_LAYER_MDI,
+  ATK_LAYER_POPUP,
+  ATK_LAYER_OVERLAY
+} AtkLayer;
 
 #define ATK_TYPE_OBJECT                           (atk_object_get_type ())
 #define ATK_OBJECT(obj)                           (G_TYPE_CHECK_INSTANCE_CAST ((obj), ATK_TYPE_OBJECT, AtkObject))
@@ -211,7 +235,7 @@ typedef struct _AtkStateSet               AtkStateSet;
 
 struct _AtkPropertyValues
 {
-  gchar  *property_name;
+  const gchar  *property_name;
   GValue old_value;
   GValue new_value;
 };
@@ -242,6 +266,7 @@ struct _AtkObject
   AtkObject *accessible_parent;
   AtkRole role;
   AtkRelationSet *relation_set;
+  AtkLayer layer;
 };
 
 struct _AtkObjectClass
@@ -285,6 +310,8 @@ struct _AtkObjectClass
    * Gets the role of the object
    */
   AtkRole                 (* get_role)            (AtkObject                 *accessible);
+  AtkLayer                (* get_layer)           (AtkObject                 *accessible);
+  gint                    (* get_mdi_zorder)      (AtkObject                 *accessible);
   /*
    * Gets the state set of the object
    */
@@ -340,8 +367,20 @@ void                      (* remove_property_change_handler)     (AtkObject
    * The signal handler which is executed  when there is a property_change 
    * signal for an object.
    */
-  gint                    (*property_change)      (AtkObject                 *accessible,
-                                                   AtkPropertyValues         *values);
+  gint                    (*property_change)      (AtkObject                  *accessible,
+                                                   AtkPropertyValues          *values);
+  /*
+   * The signal handler which is executed  when there is a state_change 
+   * signal for an object.
+   */
+  gint                    (*state_change)         (AtkObject                  *accessible,
+                                                   const gchar                *name,
+                                                   gboolean                   state_set);
+  /*
+   * The signal handler which is executed when there is a change in the
+   * visible data for an object
+   */
+  void                    (*visible_data_changed) (AtkObject                  *accessible);
 
 };
 
@@ -381,6 +420,8 @@ AtkObject*              atk_object_ref_accessible_child           (AtkObject *ac
                                                                    gint        i);
 AtkRelationSet*         atk_object_ref_relation_set               (AtkObject *accessible);
 AtkRole                 atk_object_get_role                       (AtkObject *accessible);
+AtkLayer                atk_object_get_layer                      (AtkObject *accessible);
+gint                    atk_object_get_mdi_zorder                 (AtkObject *accessible);
 AtkStateSet*            atk_object_ref_state_set                  (AtkObject *accessible);
 gint                    atk_object_get_index_in_parent            (AtkObject *accessible);
 void                    atk_object_set_name                       (AtkObject *accessible,
